/**
 * @Name PLE_COMMONATTRIBUTES_PKG.sql
 * @Author S.Deckers
 * @Date 01-JULY-2014
 * @Purpose
 * @Description
 */
 
CREATE OR REPLACE PACKAGE PLE_COMMONATTRIBUTES_PKG AS
  ---------------------------------
  -- Public data.
  ---------------------------------
  vc2OpTableName          VARCHAR2(50);
  vc2sSelect              DBMS_SQL.VARCHAR2S;
  nSelectMemberCount      NUMBER;
  vc2OpTablePrimaryKey    VARCHAR2(50);
  vc2OpTableMapfileKey    VARCHAR2(50);
  vc2OpTableOrderByKey    VARCHAR2(50);
  nOpTableSortAscending   NUMBER;

  ---------------------------------
  -- Public procedures.
  ---------------------------------
  PROCEDURE CreateOptimizedTable;

  ---------------------------------
  -- Public functions.
  ---------------------------------
  FUNCTION MapfileKey RETURN VARCHAR2;
  	PRAGMA RESTRICT_REFERENCES (MapfileKey,WNDS,WNPS,RNDS);
    
  FUNCTION PrimaryKey RETURN VARCHAR2;
  	PRAGMA RESTRICT_REFERENCES (PrimaryKey,WNDS,WNPS,RNDS);
    
  FUNCTION OrderByKey RETURN VARCHAR2;
  	PRAGMA RESTRICT_REFERENCES (OrderByKey,WNDS,WNPS,RNDS);
    
  FUNCTION SortAscending RETURN NUMBER;
  	PRAGMA RESTRICT_REFERENCES (SortAscending,WNDS,WNPS,RNDS);
    
END PLE_COMMONATTRIBUTES_PKG;
/

/**
 * @Name PLE_COMMONATTRIBUTES_PKG
 * @Date 01-07-2014
 * @Description
 */
 
create or replace PACKAGE BODY PLE_COMMONATTRIBUTES_PKG AS
  
  CVERSION  CONSTANT  VARCHAR2(30) := '10.01.00.00';
  CPACKAGE  CONSTANT  VARCHAR2(30) := REPLACE(VC2OPTABLENAME, '_OPTABLE', '_PKG');

  VC2OPTABLEEDITDATETRIGGERNAME VARCHAR2( 50) := 'OT_T_BIUR_PLE_METADTSPRT_ED';
  VC2PRIMARYKEYCONSTRAINTNAME   VARCHAR2( 50) := 'OT_P_PLE_METADTSPRT_OPTABLE';

 /**
  * @Name CREATEOPTIMIZEDTABLE
  * @Purpose 
  */
  
  PROCEDURE CREATEOPTIMIZEDTABLE IS
    VSUB  CONSTANT        VARCHAR2(61) := CPACKAGE || '.CreateOptimizedTable';
    X                     NUMBER;
    STMT2S                DBMS_SQL.VARCHAR2S;
    CUR                   INTEGER;
    ROWS_PROCESSED        INTEGER;
    I                     INTEGER;
  BEGIN
    CUR := DBMS_SQL.OPEN_CURSOR;

    SELECT COUNT(*) INTO X FROM USER_TABLES WHERE TABLE_NAME = VC2OPTABLENAME;
    
    IF X > 0 THEN          
      
      STMT2S(1) := 'TRUNCATE TABLE ' || VC2OPTABLENAME;
      DBMS_SQL.PARSE(CUR, STMT2S, 1, 1, TRUE, DBMS_SQL.V7);      
      
      STMT2S(1) := 'INSERT INTO ' || VC2OPTABLENAME;

      FOR I IN 1..NSELECTMEMBERCOUNT LOOP
        STMT2S(I + 1) := VC2SSELECT(I);
      END LOOP;

      DBMS_SQL.PARSE(CUR, STMT2S, 1, (NSELECTMEMBERCOUNT + 1), TRUE, DBMS_SQL.V7);
      ROWS_PROCESSED := DBMS_SQL.EXECUTE(CUR);

    ELSE
           
      STMT2S(1) := 'CREATE TABLE ' || VC2OPTABLENAME || ' NOLOGGING PCTFREE 0 AS';

      FOR I IN 1..NSELECTMEMBERCOUNT LOOP
        STMT2S(I + 1) := VC2SSELECT(I);
      END LOOP;

      DBMS_SQL.PARSE(CUR, STMT2S, 1, (NSELECTMEMBERCOUNT + 1), TRUE, DBMS_SQL.V7);
      
      dbms_output.put_line ( 'PARSED');
      
      STMT2S(1) := 'ALTER TABLE ' || VC2OPTABLENAME || ' ADD CONSTRAINT';
      STMT2S(2) := VC2PRIMARYKEYCONSTRAINTNAME;
      STMT2S(3) := 'PRIMARY KEY (' || VC2OPTABLEPRIMARYKEY || ')';

      DBMS_SQL.PARSE(CUR, STMT2S, 1, 3, TRUE, DBMS_SQL.V7);      
      
      STMT2S(1) := 'UPDATE ' || VC2OPTABLENAME || ' SET ';
      STMT2S(2) := 'G3E_EDITDATE = SYSDATE';

      DBMS_SQL.PARSE(CUR, STMT2S, 1, 2, TRUE, DBMS_SQL.V7);
      ROWS_PROCESSED := DBMS_SQL.EXECUTE(CUR);      
      
      STMT2S(1) := 'CREATE OR REPLACE TRIGGER '  || VC2OPTABLEEDITDATETRIGGERNAME;
      STMT2S(2) := 'BEFORE INSERT OR UPDATE ON ' || VC2OPTABLENAME;
      STMT2S(3) := 'FOR EACH ROW';
      STMT2S(4) := 'BEGIN';
      STMT2S(5) := '  :NEW.G3E_EDITDATE := SYSDATE' || CHR(59);
      STMT2S(6) := 'END' || CHR(59);

      DBMS_SQL.PARSE(CUR, STMT2S, 1, 6, TRUE, DBMS_SQL.V7);
    END IF;    
    
    DBMS_STATS.GATHER_TABLE_STATS(USER,VC2OPTABLENAME,NULL,NULL,FALSE,'FOR ALL INDEXED COLUMNS',NULL,'DEFAULT',TRUE);

    DBMS_SQL.CLOSE_CURSOR(CUR);
    COMMIT;
  EXCEPTION
    WHEN OTHERS THEN
      IF DBMS_SQL.IS_OPEN(CUR) THEN
        DBMS_SQL.CLOSE_CURSOR(CUR);
      END IF;
      RAISE;
  END CREATEOPTIMIZEDTABLE;

 /**
  * @Name MAPFILEKEY
  * @Purpose 
  */
  
  FUNCTION MAPFILEKEY RETURN VARCHAR2 IS
  BEGIN
    RETURN VC2OPTABLEMAPFILEKEY;
  END MAPFILEKEY;

 /**
  * @Name PRIMARYKEY
  * @Purpose 
  */

  FUNCTION PRIMARYKEY RETURN VARCHAR2 IS
  BEGIN
    RETURN VC2OPTABLEPRIMARYKEY;
  END PRIMARYKEY;

 /**
  * @Name ORDERBYKEY
  * @Purpose 
  */

  FUNCTION ORDERBYKEY RETURN VARCHAR2 IS
  BEGIN
    RETURN VC2OPTABLEORDERBYKEY;
  END ORDERBYKEY;

 /**
  * @Name SORTASCENDING
  * @Purpose 
  */

  FUNCTION SORTASCENDING RETURN NUMBER IS
  BEGIN
    RETURN NOPTABLESORTASCENDING;
  END SORTASCENDING;

BEGIN 
  VC2OPTABLENAME        := 'PLE_COMMONATTRIBUTES_OPTABLE';
  VC2OPTABLEPRIMARYKEY  := 'PLE_CONO';
  VC2OPTABLEMAPFILEKEY  := 'PLE_CONO';
  VC2OPTABLEORDERBYKEY  := NULL;
  NOPTABLESORTASCENDING := NULL;

  NSELECTMEMBERCOUNT := 1;

  VC2SSELECT( 1)  := 'select * FROM PLE_COMMONATTRIBUTES';

END PLE_COMMONATTRIBUTES_PKG; 
/